import os
import yaml

from eth_sdk.system.utils import resolve_path_from_base, path_exists
from eth_sdk.system import SDK_PATHS


class ConfigurationError(Exception):
    pass

class ConfigurationNotFound(Exception):
	pass


def read_config(project_root: str, network: str) -> dict:
	config_dict = None
	for path in SDK_PATHS:
		filename = path + 'config.yml'
		config_filepath = resolve_path_from_base(
			filename, project_root
		)

		if path_exists(config_filepath):
			config_missing = False
			config_dict = _load_yaml(config_filepath)
			try:
				network_config = config_dict[network]
			except KeyError:
				raise ConfigurationError(f"Network {network} wan't found in your config.yml file")
			break

		else:
			config_missing = True

	if config_missing:
		raise ConfigurationNotFound("A config.yml file wasn't found. Include one inside the eth-sdk folder or run eth-sdk --init on the root folder")
	
	return network_config


def _load_yaml(config_filepath: str) -> dict:
	with open(config_filepath, 'r') as f:
		config = yaml.safe_load(f)
		
	return config


class NetworkNotSupported(Exception):
    pass


INFURA_PROJECT_URL = '4210ba8eb536423e8ae5d774c5064fa1'
ETHERSCAN_API_KEY = 'ICZKZRP33RIZSYZJSJJ1S1ZNU64CY3KDNG'

network_ids = {
  'MAINNET': 1,
  'ROPSTEN': 3,
  'RINKEBY': 4,
  'GOERLI': 5,
  'KOVAN': 42,
  # // Arbitrum
  # ARBITRUM_ONE = 42161,
  # ARBITRUM_TESTNET = 421611,
}

network_api_urls = {
  network_ids['MAINNET']: 'https://api.etherscan.io/api',
  network_ids['ROPSTEN']: 'https://api-ropsten.etherscan.io/api',
  network_ids['RINKEBY']: 'https://api-rinkeby.etherscan.io/api',
  network_ids['GOERLI']: 'https://api-goerli.etherscan.io/api',
  network_ids['KOVAN']: 'https://api-kovan.etherscan.io/api',
  # network_ids['ARBITRUM_ONE']: 'https://api.arbiscan.io/api',
  # network_ids['ARBITRUM_TESTNET']: 'https://api-testnet.arbiscan.io/api',
}

network_api_keys = {
  network_ids['MAINNET']: ETHERSCAN_API_KEY,
  network_ids['ROPSTEN']: ETHERSCAN_API_KEY,
  network_ids['RINKEBY']: ETHERSCAN_API_KEY,
  network_ids['GOERLI']: ETHERSCAN_API_KEY,
  network_ids['KOVAN']: ETHERSCAN_API_KEY,
  # network_ids['ARBITRUM_ONE']: 'GET_KEY',
  # network_ids['ARBITRUM_TESTNET']: 'KEY',
}

rpc_providers = {
  network_ids['MAINNET']: f'https://mainnet.infura.io/v3/{INFURA_PROJECT_URL}',
  network_ids['ROPSTEN']: f'https://ropsten.infura.io/v3/{INFURA_PROJECT_URL}',
  network_ids['RINKEBY']: f'https://rinkeby.infura.io/v3/{INFURA_PROJECT_URL}',
  network_ids['GOERLI']: f'https://goerli.infura.io/v3/{INFURA_PROJECT_URL}',
  network_ids['KOVAN']: f'https://kovan.infura.io/v3/{INFURA_PROJECT_URL}',
}


def get_network_id(network_name: str) -> str:
  network_name = network_name.upper()

  try:
    network_id = network_ids[network_name]
  except KeyError:
    raise NetworkNotSupported(f"Only the following networks are currently supported: {network_ids.keys()}")

  return network_id


def get_api_url(network_name: str) -> str:
  network_id = get_network_id(network_name)

  return network_api_urls[network_id]


def get_api_key(network_name: str) -> str:
  network_id = get_network_id(network_name)

  return network_api_keys[network_id]


def get_rpc_provider(network_name: str) -> str:
  network_id = get_network_id(network_name)

  return rpc_providers[network_id]


import click
import yaml
import os


CONFIG_TEMPLATE = {
	'mainnet': {
		'contracts':  {
			'dai': '0x6B175474E89094C44Da98b954EedeAC495271d0F',
		},
		'rpc':'https://mainnet.infura.io/v3/4210ba8eb536423e8ae5d774c5064fa1',
		'etherscan': {
			'url': 'https://api.etherscan.io/api',
			'api_key': 'ICZKZRP33RIZSYZJSJJ1S1ZNU64CY3KDNG'
		},
	}
}


def cli():
	create_eth_dir()

	with open("./eth-sdk/config.yml", "w") as f:
		yaml.dump(CONFIG_TEMPLATE, f)

	click.echo("Created config.yml file")
	click.echo("eth-sdk will automatically fetch the ABI from etherscan from validated contracts in this file")


def create_eth_dir():
	try:
		os.mkdir("eth-sdk")
	except FileExistsError:
		pass


import os 


def resolve_path_from_base(filename: str, base_path: str) -> str:
	return os.path.abspath(os.path.join(base_path, filename))

	
def path_exists(config_filepath: str) -> bool:
	return os.path.exists(config_filepath)


def create_dir(path: str):
	try:
		os.mkdir(path)
	except FileExistsError:
		pass


def join_paths(path1: str, path2: str) -> str:
	return os.path.join(path1, path2)


from eth_sdk.abi_management import networks, fetch_abis
from eth_sdk.config import read_config, ConfigurationError
from web3 import Web3
import os
import dotsi
import logging

logging.basicConfig(level=logging.INFO)


class EthSdk:
	def __init__(self, network, signer):
		self.root_path = os.path.abspath(os.curdir)
		self.config = read_config(self.root_path, network)
		self.addresses = dotsi.Dict(self.config['contracts'])

		if 'rpc' in self.config:
			self.rpc = self.config['rpc']
		else:
			self.rpc = networks.get_rpc_provider(network)
			logging.warn(f"An RPC provider wasn't included in config.yml. Falling back to the default for {network}")

		if 'etherscan' in self.config:
			try:
				self.api_url = self.config['etherscan']['url']
				self.api_key = self.config['etherscan']['api_key']
			except KeyError:
				raise ConfigurationError("You should include both `url` and `api_key` under etherscan")
		else:
			self.api_url = networks.get_api_url(network)
			self.api_key = networks.get_api_key(network)
			logging.warn(f"Etherscan info wasn't included in config.yml. Falling back to the default for {network}")

		self.contract_abis = fetch_abis(self.root_path, network, self.addresses, self.api_key)
		self.w3 = Web3(Web3.HTTPProvider(self.rpc))
		self.w3.eth.default_account = signer
		self.contracts = self.build_contracts()


	def build_contracts(self):
		contracts_sdk = {}
		for contract in self.config['contracts']:
			abi = self.contract_abis[contract]
			address = self.addresses[contract]
			contracts_sdk[contract] = self.w3.eth.contract(address=address, abi=abi)

		return dotsi.Dict(contracts_sdk)


	def build_contracts_sdk(self):
		contracts_sdk = {}
		for contract in self.config['contracts']:
			abi = self.contract_abis[contract]
			address = self.addresses[contract]
			contracts_sdk[contract] = self.w3.eth.contract(address=address, abi=abi).caller

		return dotsi.Dict(contracts_sdk)


def eth_sdk(network, signer):
	sdk = EthSdk(network, signer)
	
	return sdk.build_contracts_sdk()




import requests
import json
from eth_sdk.abi_management.networks import get_api_url
from eth_sdk.system.utils import resolve_path_from_base, path_exists, create_dir, join_paths
from eth_sdk.system import SDK_PATHS


class AbiRequestError(Exception):
    pass

class InvalidApiKey(Exception):
    pass

class InvalidAddress(Exception):
    pass



def get_abi(network_name: str, contract_address: str, api_key: str) -> dict:
	api_url = get_api_url(network_name)

	url = (
	    f"{api_url}?module=contract"
		"&action=getabi"
		f"&address={contract_address}"
		f"&apikey={api_key}"
	)

	r = requests.get(url)
	
	try:
		result = r.json()['result']
	except TypeError:
		raise AbiRequestError(f"Request to {url} was unsuccessful")

	if result == 'Invalid API Key':
		raise InvalidApiKey("The provided API is not valid")

	if result == 'Invalid Address format':
		raise InvalidAddress(f"Address {contract_address} in config.yml is invalid")

	return json.loads(result)


def fetch_abis(project_root: str, network: str, contracts: dict, api_key: str) -> dict:
	contract_abis = {}
	for path in SDK_PATHS:
		sdk_path = resolve_path_from_base(
			path, project_root
		)
		if path_exists(sdk_path):
			abi_folder = join_paths(sdk_path, 'abi')
			network_folder = join_paths(abi_folder, network)

			if not path_exists(abi_folder):
				print("No ABI folder exists creating a new one")
				create_dir(abi_folder)

			if not path_exists(network_folder):
				print(f"No abi/{network} folder exists creating a new one")
				create_dir(network_folder)

			for contract in contracts:
				contract_address = contracts[contract]
				contract_abi_file = join_paths(network_folder, f"{contract}.json")

				if not path_exists(contract_abi_file):
					abi_json = get_abi(network, contract_address, api_key)
					write_json(contract_abi_file, abi_json)
				else:
					with open(contract_abi_file, "r") as f:
						abi_json = json.load(f)

				contract_abis[contract] = abi_json

			return contract_abis


def write_json(filename: str, payload: dict) -> None:
	with open(filename, 'w') as f:
		json.dump(payload, f, indent=4)


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['eth_sdk',
 'eth_sdk.abi_management',
 'eth_sdk.config',
 'eth_sdk.sdk',
 'eth_sdk.system']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'click==7.1.1',
 'dotsi>=0.0.3,<0.0.4',
 'requests>=2.26.0,<3.0.0',
 'web3>=5.24.0,<6.0.0']

entry_points = \
{'console_scripts': ['eth-sdk = eth_sdk.shell:cli']}

setup_kwargs = {
    'name': 'eth-sdk',
    'version': '0.0.1',
    'description': 'SDK to interact with EVM compatible blockchains',
    'long_description': None,
    'author': 'Arugulo',
    'author_email': 'arugulo@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.2,<4.0.0',
}


setup(**setup_kwargs)
